@@ -3,7 +3,7 @@
 // --------------------------------------------------
 
 // RUN: %libomptarget-compile-generic \
-// RUN:   -fopenmp-version=51 -DEXTENDS=BEFORE
+// RUN:   -DEXTENDS=BEFORE
 // RUN: %libomptarget-run-fail-generic 2>&1 \
 // RUN: | %fcheck-generic
 
@@ -12,39 +12,42 @@
 // --------------------------------------------------
 
 // RUN: %libomptarget-compile-generic \
-// RUN:   -fopenmp-version=51 -DEXTENDS=AFTER
+// RUN:   -DEXTENDS=AFTER
 // RUN: %libomptarget-run-fail-generic 2>&1 \
 // RUN: | %fcheck-generic
 
-
 // END.
 
 #include <stdio.h>
 
 #define BEFORE 0
-#define AFTER  1
+#define AFTER 1
 
 #define SIZE 100
 
 #if EXTENDS == BEFORE
-# define SMALL_BEG (SIZE-2)
-# define SMALL_END SIZE
-# define LARGE_BEG 0
-# define LARGE_END SIZE
+#define SMALL_BEG (SIZE - 2)
+#define SMALL_END SIZE
+#define LARGE_BEG 0
+#define LARGE_END SIZE
 #elif EXTENDS == AFTER
-# define SMALL_BEG 0
-# define SMALL_END 2
-# define LARGE_BEG 0
-# define LARGE_END SIZE
+#define SMALL_BEG 0
+#define SMALL_END 2
+#define LARGE_BEG 0
+#define LARGE_END SIZE
 #else
-# error EXTENDS undefined
+#error EXTENDS undefined
 #endif
 
-#define SMALL_SIZE (SMALL_END-SMALL_BEG)
-#define LARGE_SIZE (LARGE_END-LARGE_BEG)
+#define SMALL_SIZE (SMALL_END - SMALL_BEG)
+#define LARGE_SIZE (LARGE_END - LARGE_BEG)
 
-#define SMALL SMALL_BEG:SMALL_SIZE
-#define LARGE LARGE_BEG:LARGE_SIZE
+#define SMALL                                                                  \
+  SMALL_BEG:                                                                   \
+  SMALL_SIZE
+#define LARGE                                                                  \
+  LARGE_BEG:                                                                   \
+  LARGE_SIZE
 
 int main() {
   int arr[SIZE];
@@ -58,9 +61,9 @@ int main() {
           LARGE_SIZE * sizeof arr[0]);
 
   // CHECK-NOT: Libomptarget
-#pragma omp target data map(alloc: arr[LARGE])
+#pragma omp target data map(alloc : arr[LARGE])
   {
-#pragma omp target map(present, tofrom: arr[SMALL])
+#pragma omp target map(present, tofrom : arr[SMALL])
     ;
   }
 
@@ -69,13 +72,13 @@ int main() {
 
   // CHECK: Libomptarget message: explicit extension not allowed: host address specified is 0x{{0*}}[[#LARGE_ADDR]] ([[#LARGE_BYTES]] bytes), but device allocation maps to host at 0x{{0*}}[[#SMALL_ADDR]] ([[#SMALL_BYTES]] bytes)
   // CHECK: Libomptarget message: device mapping required by 'present' map type modifier does not exist for host address 0x{{0*}}[[#LARGE_ADDR]] ([[#LARGE_BYTES]] bytes)
-  // CHECK: Libomptarget error: Call to getOrAllocTgtPtr returned null pointer ('present' map type modifier).
+  // CHECK: Libomptarget error: Call to getTargetPointer returned null pointer ('present' map type modifier).
   // CHECK: Libomptarget error: Call to targetDataBegin failed, abort target.
   // CHECK: Libomptarget error: Failed to process data before launching the kernel.
   // CHECK: Libomptarget fatal error 1: failure of target construct while offloading is mandatory
-#pragma omp target data map(alloc: arr[SMALL])
+#pragma omp target data map(alloc : arr[SMALL])
   {
-#pragma omp target map(present, tofrom: arr[LARGE])
+#pragma omp target map(present, tofrom : arr[LARGE])
     ;
   }
 
